# -*- coding: utf-8 -*- #
# Copyright 2022 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Helpers for flags in commands working with Anthos clusters on VMware."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.calliope import arg_parsers
from googlecloudsdk.calliope.concepts import concepts
from googlecloudsdk.calliope.concepts import deps
from googlecloudsdk.command_lib.util.concepts import concept_parsers
from googlecloudsdk.core import properties


def LocationAttributeConfig():
  """Gets Google Cloud location resource attribute."""
  return concepts.ResourceParameterAttributeConfig(
      name='location',
      help_text='Google Cloud location for the {resource}.',
      fallthroughs=[
          deps.PropertyFallthrough(properties.VALUES.container_vmware.location),
      ])


def GetLocationResourceSpec():
  """Constructs and returns the Resource specification for Location."""

  return concepts.ResourceSpec(
      'gkeonprem.projects.locations',
      resource_name='location',
      projectsId=concepts.DEFAULT_PROJECT_ATTRIBUTE_CONFIG,
      locationsId=LocationAttributeConfig(),
  )


def AddLocationResourceArg(parser):
  """Adds a resource argument for Google Cloud location.

  Args:
    parser: The argparse.parser to add the resource arg to.
  """
  concept_parsers.ConceptParser.ForResource(
      '--location',
      GetLocationResourceSpec(),
      'Google Cloud location for the {resource}.',
      required=True).AddToParser(parser)


def ClusterAttributeConfig():
  return concepts.ResourceParameterAttributeConfig(
      name='cluster',
      help_text='cluster of the {resource}.',
  )


def GetClusterResourceSpec():
  return concepts.ResourceSpec(
      'gkeonprem.projects.locations.vmwareClusters',
      resource_name='cluster',
      vmwareClustersId=ClusterAttributeConfig(),
      locationsId=LocationAttributeConfig(),
      projectsId=concepts.DEFAULT_PROJECT_ATTRIBUTE_CONFIG,
  )


def AddClusterResourceArg(parser, verb, positional=True):
  """Adds a resource argument for an Anthos cluster on VMware.

  Args:
    parser: The argparse parser to add the resource arg to.
    verb: str, the verb to describe the resource, such as 'to update'.
    positional: bool, whether the argument is positional or not.
  """
  name = 'cluster' if positional else '--cluster'
  concept_parsers.ConceptParser.ForResource(
      name, GetClusterResourceSpec(), 'cluster {}'.format(verb),
      required=True).AddToParser(parser)


def NodePoolAttributeConfig():
  return concepts.ResourceParameterAttributeConfig(
      name='node_pool', help_text='node pool of the {resource}.')


def GetNodePoolResourceSpec():
  return concepts.ResourceSpec(
      'gkeonprem.projects.locations.vmwareClusters.vmwareNodePools',
      resource_name='node_pool',
      vmwareNodePoolsId=NodePoolAttributeConfig(),
      vmwareClustersId=ClusterAttributeConfig(),
      locationsId=LocationAttributeConfig(),
      projectsId=concepts.DEFAULT_PROJECT_ATTRIBUTE_CONFIG,
  )


def AddNodePoolResourceArg(parser, verb, positional=True):
  """Adds a resource argument for a VMware node pool.

  Args:
    parser: The argparse parser to add the resource arg to.
    verb: str, the verb to describe the resource, such as 'to update'.
    positional: bool, whether the argument is positional or not.
  """
  name = 'node_pool' if positional else '--node-pool'
  concept_parsers.ConceptParser.ForResource(
      name,
      GetNodePoolResourceSpec(),
      'node pool {}'.format(verb),
      required=True).AddToParser(parser)


def AdminClusterMembershipAttributeConfig():
  return concepts.ResourceParameterAttributeConfig(
      name='admin_cluster_membership',
      help_text='admin cluster membership of the {resource}, in the form of projects/PROJECT/locations/global/memberships/MEMBERSHIP. '
  )


def GetAdminClusterMembershipResourceSpec():
  return concepts.ResourceSpec(
      'gkehub.projects.locations.memberships',
      resource_name='admin_cluster_membership',
      membershipsId=AdminClusterMembershipAttributeConfig(),
      locationsId=LocationAttributeConfig(),
      projectsId=concepts.DEFAULT_PROJECT_ATTRIBUTE_CONFIG,
  )


def AddAdminClusterMembershipResourceArg(parser, positional=True):
  """Adds a resource argument for a VMware admin cluster membership.

  Args:
    parser: The argparse parser to add the resource arg to.
    positional: bool, whether the argument is positional or not.
  """
  name = 'admin_cluster_membership' if positional else '--admin-cluster-membership'
  # TODO(b/227667209): Add fallthrough from cluster location when regional
  # membership is implemented.
  concept_parsers.ConceptParser.ForResource(
      name,
      GetAdminClusterMembershipResourceSpec(),
      'membership of the admin cluster. Membership can be the membership ID or the full resource name.',
      required=True,
      flag_name_overrides={
          'location': '--admin-cluster-membership-location',
      }).AddToParser(parser)

  parser.set_defaults(admin_cluster_membership_location='global')


def AddForceUnenrollFlag(parser):
  """Adds a flag for force unenroll operation when there are existing node pools.

  Args:
    parser: The argparse parser to add the flag to.
  """
  parser.add_argument(
      '--force',
      action='store_true',
      help='If set, any child node pools will also be unenrolled. This flag is required if the cluster has any associated node pools.',
  )


def AddForceDeleteClusterFlag(parser):
  """Adds a flag for force delete cluster operation when there are existing node pools.

  Args:
    parser: The argparse parser to add the flag to.
  """
  parser.add_argument(
      '--force',
      action='store_true',
      help='If set, any node pools from the cluster will also be deleted. This flag is required if the cluster has any associated node pools.',
  )


def AddAllowMissingFlag(parser):
  """Adds a flag for delete node pool operation to return success and perform no action when there is no matching node pool.

  Args:
    parser: The argparse parser to add the flag to.
  """
  parser.add_argument(
      '--allow-missing',
      action='store_true',
      help='If set, and the Vmware Node Pool is not found, the request will succeed but no action will be taken.',
  )


def AddAllowMissingDeleteClusterFlag(parser):
  """Adds a flag for delete cluster operation to return success and perform no action when there is no matching cluster.

  Args:
    parser: The argparse parser to add the flag to.
  """
  parser.add_argument(
      '--allow-missing',
      action='store_true',
      help='If set, and the Anthos cluster on VMware is not found, the request will succeed but no action will be taken.',
  )


def AddValidationOnly(parser, hidden=False):
  """Adds a flag to only validate the request without performing the operation.

  Args:
    parser: The argparse parser to add the flag to.
    hidden: Set to False when validate-only flag is implemented in the API.
  """
  parser.add_argument(
      '--validate-only',
      action='store_true',
      help='If set, only validate the request, but do not actually perform the operation.',
      hidden=hidden,
  )


def AddImageType(parser):
  """Adds a flag to specify the node pool image type.

  Args:
    parser: The argparse parser to add the flag to.
  """
  parser.add_argument(
      '--image-type',
      required=True,
      help='Set the OS image type to use on node pool instances.',
  )


def AddReplicas(parser):
  """Adds a flag to specify the number of replicas in the node pool.

  Args:
    parser: The argparse parser to add the flag to.
  """
  parser.add_argument(
      '--replicas',
      type=int,
      help='Set the number of replicas to use on node pool instances.',
  )


def AddEnableLoadBalancer(parser):
  """Adds a flag to enable load balancer in the node pool.

  Args:
    parser: The argparse parser to add the flag to.
  """
  parser.add_argument(
      '--enable-load-balancer',
      action='store_true',
      help='If set, enable the use of load balancer on the node pool instances.',
  )


def AddAutoscaling(parser):
  """Adds a flag to specify the node pool autoscaling config.

  Args:
    parser: The argparse parser to add the flag to.
  """
  group = parser.add_argument_group('Node pool autoscaling')
  group.add_argument(
      '--min-replicas',
      required=True,
      type=int,
      help='Minimum number of replicas in the node pool',
  )
  group.add_argument(
      '--max-replicas',
      required=True,
      type=int,
      help='Maximum number of replicas in the node pool',
  )


def AddVersion(parser):
  """Adds a flag to specify the Anthos cluster on VMware version.

  Args:
    parser: The argparse parser to add the flag to.
  """
  parser.add_argument(
      '--version',
      help='Set the Anthos Cluster on VMware version for the user cluster resource',
  )


def AddServiceAddressCidrBlocks(parser):
  """Adds a flag to specify the IPv4 address ranges used in the services in the cluster.

  Args:
    parser: The argparse parser to add the flag to.
  """
  parser.add_argument(
      '--service-address-cidr-blocks',
      metavar='SERVICE_ADDRESS',
      type=arg_parsers.ArgList(
          min_length=1,
          max_length=1,
      ),
      required=True,
      help='Set the IPv4 address range for all services in the cluster.',
  )


def AddPodAddressCidrBlocks(parser):
  """Adds a flag to specify the IPv4 address ranges used in the pods in the cluster.

  Args:
    parser: The argparse parser to add the flag to.
  """
  parser.add_argument(
      '--pod-address-cidr-blocks',
      metavar='POD_ADDRESS',
      type=arg_parsers.ArgList(
          min_length=1,
          max_length=1,
      ),
      required=True,
      help='Set IPv4 address range for all pods in the cluster.',
  )


def _AddF5Config(lb_mutex_group):
  """Adds flags for F5 Big IP load balancer.

  Args:
    lb_mutex_group: The mutex group to add the flags to.
  """
  f5_config_group = lb_mutex_group.add_argument_group('F5 Big IP Configuration')
  f5_config_group.add_argument(
      '--f5-config-address',
      type=str,
      required=True,
      help='Set the F5 Big IP load balancer address.',
  )
  f5_config_group.add_argument(
      '--f5-config-partition',
      type=str,
      required=True,
      help='Set the F5 Big IP load balancer partition.',
  )
  f5_config_group.add_argument(
      '--f5-config-snat-pool',
      type=str,
      help='Set the F5 Big IP load balancer pool name if using SNAT.',
  )


def _AddMetalLbConfig(lb_mutex_group):
  """Adds flags for MetalLB load balancer.

  Args:
    lb_mutex_group: The mutex group to add the flags to.
  """
  metal_lb_config_group = lb_mutex_group.add_argument_group(
      'MetalLB Configuration')
  metal_lb_config_group.add_argument(
      '--metal-lb-config-address-pools',
      action='append',
      required=True,
      type=arg_parsers.ArgDict(
          spec={
              'pool': str,
              'addresses': arg_parsers.ArgList(),
              'avoid-buggy-ips': bool,
              'manual-assign': bool,
          },
          required_keys=[
              'pool',
              'addresses',
          ],
      ),
      help='Set the MetalLB typed load balancers configuration.',
  )


def AddLoadBalancerConfig(parser):
  """Adds a command group to set the load balancer config.

  Args:
    parser: The argparse parser to add the flag to.
  """

  lb_config_group = parser.add_argument_group(
      help='Populate the Anthos on Vmware cluster load balancer configuration.',
  )
  lb_config_group.add_argument(
      '--control-plane-vip',
      required=True,
      help='Set the VIP for the Kubernetes API of this cluster.',
  )
  lb_config_group.add_argument(
      '--ingress-vip',
      required=True,
      help='Set the VIP for ingress traffic into this cluster.',
  )

  lb_mutex_group = lb_config_group.add_mutually_exclusive_group(
      help='Populate one of the load balancers.',
      required=True,
  )

  _AddMetalLbConfig(lb_mutex_group)
  _AddF5Config(lb_mutex_group)



# -*- coding: utf-8 -*- #
# Copyright 2022 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Helper functions for constructing and validating AlloyDB cluster requests."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.command_lib.alloydb import flags


def _ConstructAutomatedBackupPolicy(alloydb_messages, args):
  """Returns the automated backup policy based on args."""
  backup_policy = alloydb_messages.AutomatedBackupPolicy()
  if args.disable_automated_backup:
    backup_policy.enabled = False
  elif args.automated_backup_days_of_week:
    backup_policy.enabled = True
    backup_policy.weeklySchedule = alloydb_messages.WeeklySchedule(
        daysOfWeek=args.automated_backup_days_of_week,
        startTimes=args.automated_backup_start_times,
    )
    if args.automated_backup_retention_count:
      backup_policy.quantityBasedRetention = (
          alloydb_messages.QuantityBasedRetention(
              count=args.automated_backup_retention_count))
    elif args.automated_backup_retention_period:
      backup_policy.timeBasedRetention = (
          alloydb_messages.TimeBasedRetention(retentionPeriod='{}s'.format(
              args.automated_backup_retention_period)))
    if args.automated_backup_window:
      backup_policy.backupWindow = '{}s'.format(args.automated_backup_window)
    kms_key = flags.GetAndValidateKmsKeyName(
        args, flag_overrides=flags.GetAutomatedBackupKmsFlagOverrides())
    if kms_key:
      encryption_config = alloydb_messages.EncryptionConfig()
      encryption_config.kmsKeyName = kms_key
      backup_policy.encryptionConfig = encryption_config
    backup_policy.location = args.region
  return backup_policy


def _ConstructPitrConfig(alloydb_messages, args):
  """Returns the pitr config based on args."""
  pitr_config = alloydb_messages.PitrConfig()
  if args.disable_pitr:
    pitr_config.enabled = False
  elif args.pitr_log_retention_window:
    pitr_config.enabled = True
    pitr_config.logRetentionWindow = '{}s'.format(
        args.pitr_log_retention_window)
  return pitr_config


def ConstructCreateRequestFromArgs(alloydb_messages, location_ref, args):
  """Returns the cluster create request based on args."""
  cluster = alloydb_messages.Cluster()
  cluster.network = args.network
  cluster.initialUser = alloydb_messages.UserPassword(
      password=args.password, user='postgres')
  kms_key = flags.GetAndValidateKmsKeyName(args)
  if kms_key:
    encryption_config = alloydb_messages.EncryptionConfig()
    encryption_config.kmsKeyName = kms_key
    cluster.encryptionConfig = encryption_config

  if args.disable_automated_backup or args.automated_backup_days_of_week:
    cluster.automatedBackupPolicy = _ConstructAutomatedBackupPolicy(
        alloydb_messages, args)

  if args.disable_pitr or args.pitr_log_retention_window:
    cluster.pitrConfig = _ConstructPitrConfig(alloydb_messages, args)

  return alloydb_messages.AlloydbProjectsLocationsClustersCreateRequest(
      cluster=cluster,
      clusterId=args.cluster,
      parent=location_ref.RelativeName())


def ConstructPatchRequestFromArgs(alloydb_messages, cluster_ref, args):
  """Returns the cluster patch request based on args."""
  update_masks = []
  cluster = alloydb_messages.Cluster()

  if (args.disable_automated_backup or args.automated_backup_days_of_week or
      args.clear_automated_backup):
    cluster.automatedBackupPolicy = _ConstructAutomatedBackupPolicy(
        alloydb_messages, args)
    update_masks.append('automated_backup_policy')

  return alloydb_messages.AlloydbProjectsLocationsClustersPatchRequest(
      name=cluster_ref.RelativeName(),
      cluster=cluster,
      updateMask=','.join(update_masks))



# -*- coding: utf-8 -*- #
# Copyright 2018 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Mappings from TextTypes to TextAttributes used by the TextTypeParser."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.core import properties
from googlecloudsdk.core.console import console_attr
from googlecloudsdk.core.console.style import ansi
from googlecloudsdk.core.console.style import text


class StyleMapping(object):
  """Mapping of TextTypes to TextAttributes."""

  def __init__(self, mappings):
    """Creates a StyleMapping object to be used by a StyledLogger.

    Args:
      mappings: (dict[TextTypes, TextAttributes]), the mapping
        to be used for this StyleMapping object.
    """
    self.mappings = mappings

  def __getitem__(self, key):
    if key in self.mappings:
      return self.mappings[key]
    return None


STYLE_MAPPINGS_BASIC = StyleMapping({
    text.TextTypes.RESOURCE_NAME: text.TextAttributes('[{}]'),
    text.TextTypes.OUTPUT: text.TextAttributes('{}'),
    text.TextTypes.USER_INPUT: text.TextAttributes('{}'),
    text.TextTypes.URI: text.TextAttributes('{}'),
    text.TextTypes.URL: text.TextAttributes('{}'),
    text.TextTypes.COMMAND: text.TextAttributes('{}'),
    text.TextTypes.INFO: text.TextAttributes('{}'),
    text.TextTypes.PT_SUCCESS: text.TextAttributes('{}'),
    text.TextTypes.PT_FAILURE: text.TextAttributes('{}'),
})


STYLE_MAPPINGS_ANSI = StyleMapping({
    text.TextTypes.RESOURCE_NAME: text.TextAttributes(
        '[{}]',
        color=ansi.Colors.BLUE,
        attrs=[]),
    text.TextTypes.OUTPUT: text.TextAttributes(
        '[{}]',
        color=ansi.Colors.BLUE,
        attrs=[]),
    text.TextTypes.USER_INPUT: text.TextAttributes(
        '{}',
        color=ansi.Colors.CYAN,
        attrs=[ansi.Attrs.BOLD]),
    text.TextTypes.URI: text.TextAttributes(
        '{}',
        color=None,
        attrs=[]),
    text.TextTypes.URL: text.TextAttributes(
        '{}',
        color=None,
        attrs=[ansi.Attrs.UNDERLINE]),
    text.TextTypes.COMMAND: text.TextAttributes(
        '{}',
        color=ansi.Colors.GREEN,
        attrs=[]),
    text.TextTypes.INFO: text.TextAttributes(
        '{}',
        color=ansi.Colors.YELLOW,
        attrs=[]),
    text.TextTypes.PT_SUCCESS: text.TextAttributes(
        '{}', color=ansi.Colors.GREEN),
    text.TextTypes.PT_FAILURE: text.TextAttributes(
        '{}', color=ansi.Colors.RED),
})


STYLE_MAPPINGS_ANSI_256 = StyleMapping({
    text.TextTypes.RESOURCE_NAME: text.TextAttributes(
        '[{}]',
        color=ansi.Colors.COLOR_33,
        attrs=[]),
    text.TextTypes.OUTPUT: text.TextAttributes(
        '[{}]',
        color=ansi.Colors.COLOR_33,
        attrs=[]),
    text.TextTypes.USER_INPUT: text.TextAttributes(
        '{}',
        color=ansi.Colors.COLOR_81,
        attrs=[ansi.Attrs.BOLD]),
    text.TextTypes.URI: text.TextAttributes(
        '{}',
        color=None,
        attrs=[]),
    text.TextTypes.URL: text.TextAttributes(
        '{}',
        color=None,
        attrs=[ansi.Attrs.UNDERLINE]),
    text.TextTypes.COMMAND: text.TextAttributes(
        '{}',
        color=ansi.Colors.COLOR_34,
        attrs=[]),
    text.TextTypes.INFO: text.TextAttributes(
        '{}',
        color=ansi.Colors.COLOR_167,
        attrs=[]),
    text.TextTypes.PT_SUCCESS: text.TextAttributes(
        '{}', color=ansi.Colors.GREEN),
    text.TextTypes.PT_FAILURE: text.TextAttributes(
        '{}', color=ansi.Colors.RED),
})


STYLE_MAPPINGS_TESTING = StyleMapping(dict([
    (text_type, text.TextAttributes('[{{}}]({})'.format(text_type.name)))
    for text_type in [
        text.TextTypes.RESOURCE_NAME,
        text.TextTypes.OUTPUT,
        text.TextTypes.USER_INPUT,
        text.TextTypes.URI,
        text.TextTypes.URL,
        text.TextTypes.COMMAND,
        text.TextTypes.INFO,
        text.TextTypes.PT_SUCCESS,
        text.TextTypes.PT_FAILURE]]))


def GetStyleMappings(console_attributes=None):
  """Gets the style mappings based on the console and user properties."""
  console_attributes = console_attributes or console_attr.GetConsoleAttr()
  is_screen_reader = properties.VALUES.accessibility.screen_reader.GetBool()
  if properties.VALUES.core.color_theme.Get() == 'testing':
    return STYLE_MAPPINGS_TESTING
  elif (not is_screen_reader and
        console_attributes.SupportsAnsi() and
        properties.VALUES.core.color_theme.Get() != 'off'):
    if console_attributes._term == 'xterm-256color':  # pylint: disable=protected-access
      return STYLE_MAPPINGS_ANSI_256
    else:
      return STYLE_MAPPINGS_ANSI
  else:
    return STYLE_MAPPINGS_BASIC



# -*- coding: utf-8 -*- #
# Copyright 2016 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Console Prompter for compute scopes."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

import operator

from googlecloudsdk.command_lib.compute import scope as compute_scope
from googlecloudsdk.core import log
from googlecloudsdk.core import properties
from googlecloudsdk.core.console import console_io
from googlecloudsdk.core.credentials import gce as c_gce
from googlecloudsdk.core.util import text


def PromptForScope(resource_name, underspecified_names,
                   scopes, default_scope, scope_lister):
  """Prompt user to specify a scope.

  Args:
    resource_name: str, human readable name of the resource.
    underspecified_names: list(str), names which lack scope context.
    scopes: list(compute_scope.ScopeEnum), scopes to query for.
    default_scope: compute_scope.ScopeEnum, force this scope to be used.
    scope_lister: func(scopes, underspecified_names)->[str->[str]], callback to
        provide possible values for each scope.
  Returns:
    tuple of chosen scope_enum and scope value.
  """
  implicit_scope = default_scope
  if len(scopes) == 1:
    implicit_scope = scopes[0]

  if implicit_scope:
    suggested_value = _GetSuggestedScopeValue(implicit_scope)
    if suggested_value is not None:
      if _PromptDidYouMeanScope(resource_name, underspecified_names,
                                implicit_scope, suggested_value):
        return implicit_scope, suggested_value

  if not scope_lister:
    return None, None

  scope_value_choices = scope_lister(
      # Sort to make it deterministic.
      sorted(scopes, key=operator.attrgetter('name')),
      underspecified_names)

  choice_names, choice_mapping = _FormatScopeValueChoices(scope_value_choices)

  if len(choice_mapping) == 1:
    suggested_resource_scope_enum = choice_mapping[0][0]
    suggested_resource = choice_mapping[0][1]
    _PromptSuggestedScopeChoice(resource_name, underspecified_names,
                                suggested_resource_scope_enum,
                                suggested_resource)
    return suggested_resource_scope_enum, suggested_resource

  resource_scope_enum, scope_value = _PromptWithScopeChoices(
      resource_name, underspecified_names, scope_value_choices, choice_names,
      choice_mapping)

  return resource_scope_enum, scope_value


def _PromptDidYouMeanScope(resource_name, underspecified_names, scope_enum,
                           suggested_resource):
  """Prompts "did you mean <scope>".  Returns str or None."""
  if scope_enum == compute_scope.ScopeEnum.GLOBAL:
    message = 'Did you mean [{0}] for {1}: [{2}]'.format(
        scope_enum.flag_name, resource_name, ', '.join(underspecified_names))
  else:
    message = 'Did you mean {0} [{1}] for {2}: [{3}]'.format(
        scope_enum.flag_name, suggested_resource,
        resource_name, ', '.join(underspecified_names))
  return console_io.PromptContinue(prompt_string=message, default=True,
                                   throw_if_unattended=True)


def _PromptSuggestedScopeChoice(resource_name, underspecified_names, scope_enum,
                                suggested_resource):
  if scope_enum == compute_scope.ScopeEnum.GLOBAL:
    log.status.Print('No scope specified. Using [{0}] for {1}: [{2}].'.format(
        scope_enum.flag_name,
        text.Pluralize(len(underspecified_names), resource_name),
        ', '.join(underspecified_names)))
  else:
    log.status.Print('No {0} specified. Using {0} [{1}] for {2}: [{3}].'.format(
        scope_enum.flag_name, suggested_resource,
        text.Pluralize(len(underspecified_names), resource_name),
        ', '.join(underspecified_names)))


def _FormatScopeValueChoices(scope_value_choices):
  """Formats scope value choices for prompting and adds deprecation states."""
  choice_names, choice_mapping = [], []
  for scope in sorted(list(scope_value_choices.keys()),
                      key=operator.attrgetter('flag_name')):
    for choice_resource in sorted(scope_value_choices[scope],
                                  key=operator.attrgetter('name')):
      deprecated = getattr(choice_resource, 'deprecated', None)
      if deprecated is not None:
        choice_name = '{0} ({1})'.format(
            choice_resource.name, deprecated.state)
      else:
        choice_name = choice_resource.name

      if len(scope_value_choices) > 1:
        if choice_name:
          choice_name = '{0}: {1}'.format(scope.flag_name, choice_name)
        else:
          choice_name = scope.flag_name

      choice_mapping.append((scope, choice_resource.name))
      choice_names.append(choice_name)

  return choice_names, choice_mapping


def _PromptWithScopeChoices(resource_name, underspecified_names,
                            scope_value_choices, choice_names, choice_mapping):
  """Queries user to choose scope and its value."""
  title = ('For the following {0}:\n {1}\n'
           .format(text.Pluralize(len(underspecified_names), resource_name),
                   '\n '.join('- [{0}]'.format(n)
                              for n in sorted(underspecified_names))))
  flags = ' or '.join(
      sorted([s.prefix + s.flag_name for s in scope_value_choices.keys()]))

  idx = console_io.PromptChoice(
      options=choice_names, message='{0}choose {1}:'.format(title, flags))
  if idx is None:
    return None, None
  else:
    return choice_mapping[idx]


def _GetSuggestedScopeValue(scope):
  if scope == compute_scope.ScopeEnum.ZONE:
    return _GetGCEZone()
  if scope == compute_scope.ScopeEnum.REGION:
    return _GetGCERegion()
  return True


def _GetGCERegion():
  if properties.VALUES.core.check_gce_metadata.GetBool():
    return c_gce.Metadata().Region()
  return None


def _GetGCEZone():
  if properties.VALUES.core.check_gce_metadata.GetBool():
    return c_gce.Metadata().Zone()
  return None



# -*- coding: utf-8 -*- #
# Copyright 2020 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Traffic-specific printer and functions for generating traffic formats."""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

from googlecloudsdk.api_lib.run import service
from googlecloudsdk.api_lib.run import traffic_pair
from googlecloudsdk.command_lib.run import platforms
from googlecloudsdk.core.console import console_attr
from googlecloudsdk.core.resource import custom_printer_base as cp


TRAFFIC_PRINTER_FORMAT = 'traffic'
_INGRESS_UNSPECIFIED = '-'


def _GetIngress(record):
  """Gets the ingress traffic allowed to call the service."""
  if platforms.GetPlatform() == platforms.PLATFORM_MANAGED:
    spec_ingress = record.annotations.get(service.INGRESS_ANNOTATION)
    status_ingress = record.annotations.get(service.INGRESS_STATUS_ANNOTATION)
    if spec_ingress == status_ingress:
      return spec_ingress
    else:
      spec_ingress = spec_ingress or _INGRESS_UNSPECIFIED
      status_ingress = status_ingress or _INGRESS_UNSPECIFIED
      return '{} (currently {})'.format(spec_ingress, status_ingress)
  elif (record.labels.get(
      service.ENDPOINT_VISIBILITY) == service.CLUSTER_LOCAL):
    return service.INGRESS_INTERNAL
  else:
    return service.INGRESS_ALL


def _GetTagAndStatus(tag):
  """Returns the tag with padding and an adding/removing indicator if needed."""
  if tag.inSpec and not tag.inStatus:
    return '  {} (Adding):'.format(tag.tag)
  elif not tag.inSpec and tag.inStatus:
    return '  {} (Deleting):'.format(tag.tag)
  else:
    return '  {}:'.format(tag.tag)


def _TransformTrafficPair(pair):
  """Transforms a single TrafficTargetPair into a marker class structure."""
  console = console_attr.GetConsoleAttr()
  return (pair.displayPercent, console.Emphasize(pair.displayRevisionId),
          cp.Table([('', _GetTagAndStatus(t), t.url) for t in pair.tags]))


def _TransformTrafficPairs(traffic_pairs, service_url, service_ingress=None):
  """Transforms a List[TrafficTargetPair] into a marker class structure."""
  traffic_section = cp.Section(
      [cp.Table(_TransformTrafficPair(p) for p in traffic_pairs)])
  route_section = [cp.Labeled([('URL', service_url)])]
  if service_ingress is not None:
    route_section.append(cp.Labeled([('Ingress', service_ingress)]))
  route_section.append(cp.Labeled([('Traffic', traffic_section)]))
  return cp.Section(route_section, max_column_width=60)


def TransformRouteFields(service_record):
  """Transforms a service's route fields into a marker class structure to print.

  Generates the custom printing format for a service's url, ingress, and traffic
  using the marker classes defined in custom_printer_base.

  Args:
    service_record: A Service object.

  Returns:
    A custom printer marker object describing the route fields print format.
  """
  no_status = service_record.status is None
  traffic_pairs = traffic_pair.GetTrafficTargetPairs(
      service_record.spec_traffic, service_record.status_traffic,
      service_record.is_managed,
      (_INGRESS_UNSPECIFIED
       if no_status else service_record.status.latestReadyRevisionName))
  return _TransformTrafficPairs(traffic_pairs,
                                '' if no_status else service_record.status.url,
                                _GetIngress(service_record))


class TrafficPrinter(cp.CustomPrinterBase):
  """Prints a service's traffic in a custom human-readable format."""

  def Print(self, resources, single=False, intermediate=False):
    """Overrides ResourcePrinter.Print to set single=True."""
    # The update-traffic command returns a List[TrafficTargetPair] as its
    # result. In order to print the custom human-readable format, this printer
    # needs to process all records in the result at once (to compute column
    # widths). By default, ResourcePrinter interprets a List[*] as a list of
    # separate records and passes the contents of the list to this printer
    # one-by-one. Setting single=True forces ResourcePrinter to treat the
    # result as one record and pass the entire list to this printer in one call.
    super(TrafficPrinter, self).Print(resources, True, intermediate)

  def Transform(self, record):
    """Transforms a List[TrafficTargetPair] into a marker class format."""
    if record:
      service_url = record[0].serviceUrl
    else:
      service_url = ''
    return _TransformTrafficPairs(record, service_url)



# -*- coding: utf-8 -*- #
# Copyright 2020 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Helpers to add flags to kuberun commands."""
from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

import abc

from googlecloudsdk.calliope import arg_parsers
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.util.concepts import concept_parsers
from googlecloudsdk.core import exceptions
from googlecloudsdk.core import log
from googlecloudsdk.core import properties
from googlecloudsdk.core.console import console_io

import six

_VISIBILITY_MODES = {
    'internal': 'Visible only within the cluster.',
    'external': 'Visible from outside the cluster.',
}

CONNECTION_GKE = 'GKE'
CONNECTION_KUBECONFIG = 'KUBECONFIG'


class ConfigurationError(exceptions.Error):
  """Indicates an error in configuration."""


class BinaryCommandFlag(six.with_metaclass(abc.ABCMeta, object)):
  """Informal interface for flags that get passed through to an underlying binary."""

  @abc.abstractmethod
  def AddToParser(self, parser):
    """Adds this argument to the given parser.

    Args:
      parser: The argparse parser.
    """
    pass

  @abc.abstractmethod
  def FormatFlags(self, args):
    """Return flags in a format that can be passed to the underlying binary."""
    pass


class StringFlag(BinaryCommandFlag):
  """A flag that takes a string value that is just passed directly through to the binary."""

  def __init__(self, name, **kwargs):
    super(StringFlag, self).__init__()
    self.arg = base.Argument(name, **kwargs)

  def AddToParser(self, parser):
    return self.arg.AddToParser(parser)

  def FormatFlags(self, args):
    dest_name = _GetDestNameForFlag(self.arg.name)
    if args.IsSpecified(dest_name):
      return [self.arg.name, str(getattr(args, dest_name))]
    return []


class StringListFlag(BinaryCommandFlag):
  """A flag that takes in a string list value passed directly through to the binary."""

  def __init__(self, name, *args, **kwargs):
    super(StringListFlag, self).__init__()
    self.arg = base.Argument(name, *args, type=arg_parsers.ArgList(), **kwargs)

  def AddToParser(self, parser):
    return self.arg.AddToParser(parser)

  def FormatFlags(self, args):
    dest_name = _GetDestNameForFlag(self.arg.name)
    if args.IsSpecified(dest_name):
      return [self.arg.name, ','.join(getattr(args, dest_name))]
    return []


class BooleanFlag(BinaryCommandFlag):
  """Encapsulates a boolean flag that can be either --<flag> or --no-<flag>."""

  def __init__(self, name, **kwargs):
    super(BooleanFlag, self).__init__()
    self.arg = base.Argument(
        name, action=arg_parsers.StoreTrueFalseAction, **kwargs)

  def AddToParser(self, parser):
    return self.arg.AddToParser(parser)

  def FormatFlags(self, args):
    base_flag = self.arg.name.replace('--', '')
    bool_flags = [self.arg.name, '--no-' + base_flag]
    return [f for f in bool_flags if f in args.GetSpecifiedArgNames()]


class BasicFlag(BinaryCommandFlag):
  """Encapsulates a flag that is passed through as-is when present."""

  def __init__(self, name, **kwargs):
    super(BasicFlag, self).__init__()
    self.arg = base.Argument(name, default=False, action='store_true', **kwargs)

  def AddToParser(self, parser):
    return self.arg.AddToParser(parser)

  def FormatFlags(self, args):
    dest_name = _GetDestNameForFlag(self.arg.name)
    if args.IsSpecified(dest_name):
      return [self.arg.name]
    return []


class FlagGroup(BinaryCommandFlag):
  """Encapsulates multiple flags that are logically added together."""

  def __init__(self, first, second, *args):
    """Create a new flag group.

    At least two flags must be specified.

    Args:
      first: the first flag in the group
      second: the second flag in the group
      *args: additional flags in the group
    """
    super(FlagGroup, self).__init__()
    all_flags = [first, second]
    all_flags.extend(args)
    self._flags = all_flags

  def AddToParser(self, parser):
    for f in self._flags:
      f.AddToParser(parser)

  def FormatFlags(self, args):
    all_flags = []
    for f in self._flags:
      all_flags.extend(f.FormatFlags(args))
    return all_flags


def NamespaceFlag():
  return StringFlag('--namespace', help='Kubernetes namespace to operate in.')


class NamespaceFlagGroup(BinaryCommandFlag):
  """Encapsulates logic for handling the mutually-exclusive flags --namespace and --all-namespaces."""

  def AddToParser(self, parser):
    mutex_group = parser.add_mutually_exclusive_group()
    NamespaceFlag().AddToParser(mutex_group)
    mutex_group.add_argument(
        '--all-namespaces',
        default=False,
        action='store_true',
        help='List the requested object(s) across all namespaces.')

  def FormatFlags(self, args):
    if args.IsSpecified('all_namespaces'):
      return ['--all-namespaces']
    return NamespaceFlag().FormatFlags(args)


class ClusterConnectionFlags(BinaryCommandFlag):
  """Encapsulates logic for handling flags used for connecting to a cluster."""

  def AddToParser(self, parser):
    # Import done here to avoid circular import
    # pylint: disable=g-import-not-at-top
    from googlecloudsdk.command_lib.kuberun import resource_args

    mutex_group = parser.add_mutually_exclusive_group()
    concept_parsers.ConceptParser(
        [resource_args.CLUSTER_PRESENTATION]).AddToParser(mutex_group)
    kubeconfig_group = mutex_group.add_group()
    kubeconfig_group.add_argument(
        '--context',
        help='Name of the context in your kubectl config file to use for '
        'connecting. Cannot be specified together with --cluster and '
        '--cluster-location.')
    kubeconfig_group.add_argument(
        '--kubeconfig',
        help='Absolute path to your kubectl config file. Cannot be specified '
        'together with --cluster and --cluster-location.')
    kubeconfig_group.add_argument(
        '--use-kubeconfig',
        default=False,
        action='store_true',
        help='Use the kubectl config to connect to the cluster. If '
        '--kubeconfig is not also provided, the colon- or semicolon-delimited '
        'list of paths specified by $KUBECONFIG will be used. If $KUBECONFIG '
        'is unset, this defaults to ~/.kube/config. Cannot be specified '
        'together with --cluster and --cluster-location.')

  def FormatFlags(self, args):
    exec_args = []
    connection = ClusterConnectionMethod(args)

    if connection == CONNECTION_GKE:
      cluster_ref = ParseClusterRefOrPromptUser(args)
      exec_args.extend(['--cluster', cluster_ref.SelfLink()])

    elif connection == CONNECTION_KUBECONFIG:
      # The kuberun binary defaults to using kubeconfig. If none of these flags
      # end up getting passed, that is OK and the execution will still use the
      # default kubeconfig.
      kubeconfig, context = KubeconfigPathAndContext(args)
      if kubeconfig:
        exec_args.extend(['--kubeconfig', kubeconfig])
      if context:
        exec_args.extend(['--context', context])

    return exec_args


def ClusterConnectionMethod(args):
  """Returns the connection method resulting from args and configuration.

  This functionality is broken out so that it can be used as a means to
  determine whether the user should be prompted to select a cluster, although
  the user is not prompted as part of this function's execution.

  Args:
    args: Namespace of parsed args

  Returns:
     Constant, one of CONNECTION_GKE or CONNECTION_KUBECONFIG.

  Raises:
    ConfigurationError: when the configuration is invalid.
  """
  # Explicit flags passed in take precedence. If no explicit flags are provided,
  # the assumption is "GKE" unless use_kubeconfig or kubeconfig is configured.

  # Explicit args
  if args.IsSpecified('cluster') or args.IsSpecified('cluster_location'):
    return CONNECTION_GKE
  if (args.IsSpecified('use_kubeconfig') or args.IsSpecified('kubeconfig') or
      args.IsSpecified('context')):
    return CONNECTION_KUBECONFIG

  configured_kubeconfig_options = _ExplicitlySetProperties(
      ['kubeconfig', 'use_kubeconfig', 'context'])
  configured_gke_options = _ExplicitlySetProperties(
      ['cluster', 'cluster_location'])

  if configured_kubeconfig_options and configured_gke_options:
    raise ConfigurationError(
        'Multiple cluster connection options are configured. '
        'To remove one of the options, run `{}` or `{}`.'.format(
            _UnsetCommandsAsString(configured_kubeconfig_options),
            _UnsetCommandsAsString(configured_gke_options)))

  if configured_kubeconfig_options:
    return CONNECTION_KUBECONFIG
  return CONNECTION_GKE


def KubeconfigPathAndContext(args):
  """Returns a 2-tuple of (kubeconfig, context).

  The kubeconfig path and context returned will be those specified in args
  or those coming from properties. Missing values for kubeconfig or context
  will be returned as None values.

  Args:
    args: Parsed argument context object

  Returns:
    2-tuple of (kubeconfig, context) where the kubeconfig is the path to the
    a kubeconfig file and the context is the name of the context to be used.
  """
  kubeconfig = (args.kubeconfig if args.IsSpecified('kubeconfig') else
                properties.VALUES.kuberun.kubeconfig.Get())
  context = (args.context if args.IsSpecified('context') else
             properties.VALUES.kuberun.context.Get())
  return kubeconfig, context


def ParseClusterRefOrPromptUser(args):
  """Returns a ref to a GKE cluster based on args or prompting.

  Args:
    args: Parsed argument context object

  Returns:
    A Resource object representing the cluster

  Raises:
    ConfigurationError: when the user has not specified a cluster
      connection method and can't be prompted.
  """
  cluster_ref = args.CONCEPTS.cluster.Parse()
  if not cluster_ref:
    raise ConfigurationError(
        'You must specify a cluster in a given location. '
        'Either use the `--cluster` and `--cluster-location` flags '
        'or set the kuberun/cluster and kuberun/cluster_location properties.')
  return cluster_ref


def _UnsetCommandsAsString(names):
  commands = ['gcloud config unset kuberun/{}'.format(name) for name in names]
  return ' && '.join(commands)


def _ExplicitlySetProperties(property_names):
  return [name for name in property_names if
          getattr(properties.VALUES.kuberun, name).IsExplicitlySet()]


class TrafficFlags(BinaryCommandFlag):
  """Encapsulates flags to configure traffic routes to the service."""

  def __init__(self):
    super(TrafficFlags, self).__init__()
    self.to_revisions_flag = StringListFlag(
        '--to-revisions',
        metavar='REVISION-NAME=PERCENTAGE',
        help='Comma-separated list of traffic assignments in the form '
        'REVISION-NAME=PERCENTAGE. REVISION-NAME must be the name for a revision '
        'for the service as returned by \'gcloud kuberun core revisions list\'. '
        'PERCENTAGE must be an integer percentage between 0 and 100 inclusive. '
        'E.g. service-nw9hs=10,service-nw9hs=20 Up to 100 percent of traffic may '
        'be assigned. If 100 percent of traffic is assigned, the Service traffic '
        'is updated as specified. If under 100 percent of traffic is assigned, '
        'the Service traffic is updated as specified for revisions with '
        'assignments and traffic is scaled up or down down proportionally as '
        'needed for revision that are currently serving traffic but that do not '
        'have new assignments. For example assume revision-1 is serving 40 '
        'percent of traffic and revision-2 is serving 60 percent. If revision-1 '
        'is assigned 45 percent of traffic and no assignment is made for '
        'revision-2, the service is updated with revsion-1 assigned 45 percent '
        'of traffic and revision-2 scaled down to 55 percent. You can use '
        '"LATEST" as a special revision name to always put the given percentage '
        'of traffic on the latest ready revision.')

  def AddToParser(self, parser):
    mutex_group = parser.add_mutually_exclusive_group(required=True)
    mutex_group.add_argument(
        '--to-latest',
        default=False,
        action='store_true',
        help='If true, assign 100 percent of traffic to the \'latest\' revision '
        'of this service. Note that when a new revision is created, it will '
        'become the \'latest\' and traffic will be directed to it. Defaults to '
        'False. Synonymous with `--to-revisions=LATEST=100`.')

    self.to_revisions_flag.AddToParser(mutex_group)

  def FormatFlags(self, args):
    if args.IsSpecified('to_latest'):
      return ['--to-latest']
    elif args.IsSpecified('to_revisions'):
      return self.to_revisions_flag.FormatFlags(args)


def CreateIfMissingFlag():
  return BasicFlag(
      '--create-if-missing',
      help="Creates the service if it's missing. If set, `--image` must be set as well."
  )


def RevisionListServiceFlag():
  return StringFlag(
      '--service', help='List the revisions for only the specified service.')


def ServiceListServiceFlag():
  return StringFlag(
      '--service', help='List only the specified service.')


def CommonServiceFlags(is_create=False):
  return FlagGroup(NamespaceFlag(), ImageFlag(required=is_create), CPUFlag(),
                   GPUFlag(), MemoryFlag(), PortFlag(), Http2Flag(),
                   ConcurrencyFlag(), EntrypointFlags(), ScalingFlags(),
                   LabelsFlags(set_flag_only=is_create),
                   ConfigMapFlags(set_flag_only=is_create),
                   SecretsFlags(set_flag_only=is_create),
                   EnvVarsFlags(set_flag_only=is_create), ConnectivityFlag(),
                   ServiceAccountFlag(), RevisionSuffixFlag(), TimeoutFlag())


def ImageFlag(required=False):
  return StringFlag(
      '--image',
      help='Name of the container image to deploy (e.g. gcr.io/cloudrun/hello:latest).',
      required=required)


def CPUFlag():
  return StringFlag(
      '--cpu',
      help='CPU limit, in Kubernetes cpu units, for the resource. Ex: .5, 500m, 2. To unset this field, pass the special value "default".'
  )


def GPUFlag():
  return StringFlag(
      '--gpu',
      help='GPU limit, in Kubernetes gpu units, for the resource. Ex: 1, 2. To unset this field, pass the special value "default".'
  )


def MemoryFlag():
  return StringFlag(
      '--memory',
      help='Memory limit for the resource. Ex: 1Gi, 512Mi. To unset this field, pass the special value "default".'
  )


def PortFlag():
  return StringFlag(
      '--port',
      help='Container port to receive requests at. Also sets the $PORT environment variable. Must be a number between 1 and 65535, inclusive. To unset this field, pass the special value "default".'
  )


def Http2Flag():
  return BooleanFlag(
      '--use-http2',
      help='If true, uses HTTP/2 for connections to the service.')


def ConcurrencyFlag():
  return StringFlag(
      '--concurrency',
      help='Maximum number of concurrent requests allowed per container instance. If concurrency is unspecified, any number of concurrent requests are allowed. To unset this field, provide the special value "default".'
  )


def EntrypointFlags():
  """Encapsulate flags for customizing container command."""
  args_flag = StringListFlag(
      '--args',
      metavar='ARG',
      help='Comma-separated arguments passed to the command run by the '
      "container image. If not specified and no '--command' is provided, the "
      "container image's default Cmd is used. Otherwise, if not specified, no "
      'arguments are passed. To reset this field to its default, pass an empty '
      'string.')
  command_flag = StringListFlag(
      '--command',
      metavar='COMMAND',
      help='Entrypoint for the container image. If not specified, the '
      "container image's default Entrypoint is run. To reset this field to its "
      'default, pass an empty string.')
  return FlagGroup(args_flag, command_flag)


def ScalingFlags():
  """Encapsulates flags related to scaling instances."""
  init_instances_flag = StringFlag(
      '--init-instances',
      type=arg_parsers.BoundedInt(lower_bound=0),
      help='Initial number of container instances of the Service to run. '
      'Can be 0 or a positive number. '
      'Pass "default" to unset this field and use the platform default.')
  min_instances_flag = StringFlag(
      '--min-instances',
      help='Minimum number of container instances of the Service to run '
      'or "default" to remove any minimum.')
  max_instances_flag = StringFlag(
      '--max-instances',
      help='Maximum number of container instances of the Service to run. '
      'Use "default" to unset the limit and use the platform default.')
  return FlagGroup(init_instances_flag, min_instances_flag, max_instances_flag)


class ResourceListFlagGroup(BinaryCommandFlag):
  """Encapsulates create/set/update/remove key-value flags."""

  def __init__(
      self,
      name,
      help=None,  # pylint: disable=redefined-builtin
      help_name=None,
      set_flag_only=False):
    """Create a new resource list flag group.

    Args:
      name: the name to be used in the flag names (e.g. "config-maps")
      help: supplementary help text that explains the key-value pairs
      help_name: the resource name to use in help text if different from `name`
      set_flag_only: whether to just add the set-{name} flag.
    """
    super(ResourceListFlagGroup, self).__init__()
    self.set_flag_only = set_flag_only
    self.help = help if not help else help + '\n\n'
    help_name = name if help_name is None else help_name

    pairs_help = 'List of key-value pairs to set as {}.'.format(help_name)
    set_help = pairs_help
    if set_flag_only:
      if help:
        set_help += '\n\n' + help
    else:
      set_help += ' All existing {} will be removed first.'.format(help_name)

    self.clear_flag = BasicFlag(
        '--clear-{}'.format(name),
        help='If true, removes all {}.'.format(help_name))
    self.set_flag = StringListFlag(
        '--set-{}'.format(name), metavar='KEY=VALUE', help=set_help)
    self.remove_flag = StringListFlag(
        '--remove-{}'.format(name),
        metavar='KEY',
        help='List of {} to be removed.'.format(help_name))
    update_aliases = []
    if name == 'labels':
      # Added for compatibility reasons with gcloud run services update
      update_aliases.append('--labels')
    self.update_flag = StringListFlag(
        '--update-{}'.format(name),
        *update_aliases,
        metavar='KEY=VALUE',
        help=pairs_help)

  def AddToParser(self, parser):
    if self.set_flag_only:
      self.set_flag.AddToParser(parser)
      return

    mutex_group = parser.add_mutually_exclusive_group(help=self.help)
    self.clear_flag.AddToParser(mutex_group)
    self.set_flag.AddToParser(mutex_group)
    update_group = mutex_group.add_group(
        help='Only `{update}` and `{remove}` can be used together. '
        'If both are specified, `{remove}` will be applied first.'.format(
            update=self.update_flag.arg.name, remove=self.remove_flag.arg.name))
    self.remove_flag.AddToParser(update_group)
    self.update_flag.AddToParser(update_group)

  def FormatFlags(self, args):
    if self.set_flag_only:
      return self.set_flag.FormatFlags(args)
    return (self.clear_flag.FormatFlags(args) +
            self.set_flag.FormatFlags(args) +
            self.remove_flag.FormatFlags(args) +
            self.update_flag.FormatFlags(args))


def LabelsFlags(set_flag_only=False):
  return ResourceListFlagGroup(name='labels', set_flag_only=set_flag_only)


def ConfigMapFlags(set_flag_only=False):
  return ResourceListFlagGroup(
      name='config-maps',
      set_flag_only=set_flag_only,
      help=(
          'Specify config maps to mount or provide as environment variables. '
          "Keys starting with a forward slash '/' are mount paths. "
          'All other keys correspond to environment variables. '
          'The values associated with each of these should be in the form '
          'CONFIG_MAP_NAME:KEY_IN_CONFIG_MAP; you may omit the key within the '
          'config map to specify a mount of all keys within the config map. '
          'For example: '
          '`--set-config-maps=/my/path=myconfig,ENV=otherconfig:key.json` will '
          "create a volume with config map 'myconfig' and mount that volume at "
          "'/my/path'. Because no config map key was specified, all keys in "
          "'myconfig' will be included. "
          'An environment variable named ENV will also be created whose value '
          "is the value of 'key.json' in 'otherconfig'."))


def SecretsFlags(set_flag_only=False):
  return ResourceListFlagGroup(
      name='secrets',
      set_flag_only=set_flag_only,
      help=(
          'Specify secrets to mount or provide as environment variables. '
          "Keys starting with a forward slash '/' are mount paths. "
          'All other keys correspond to environment variables. '
          'The values associated with each of these should be in the form '
          'SECRET_NAME:KEY_IN_SECRET; you may omit the key within the secret '
          'to specify a mount of all keys within the secret. '
          'For example: '
          '`--set-secrets=/my/path=mysecret,ENV=othersecret:key.json` will '
          "create a volume with secret 'mysecret' and mount that volume at "
          "'/my/path'. Because no secret key was specified, all keys in "
          "'mysecret' will be included. "
          'An environment variable named ENV will also be created whose value '
          "is the value of 'key.json' in 'othersecret'."))


def EnvVarsFlags(set_flag_only=False):
  return ResourceListFlagGroup(
      name='env-vars',
      help_name='environment variables',
      set_flag_only=set_flag_only)


def ConnectivityFlag():
  return StringFlag(
      '--connectivity',
      help='Defaults to \'external\'. If \'external\', the service can be '
      'invoked through the internet, in addition to through the cluster '
      'network.',
      choices=_VISIBILITY_MODES)


def NoTrafficFlag():
  return BasicFlag(
      '--no-traffic',
      help='If set, any traffic assigned to the LATEST revision will be '
      'assigned to the specific revision bound to LATEST before the '
      'deployment. This means the revision being deployed will not receive '
      'traffic. After a deployment with this flag, the LATEST revision will '
      'not receive traffic on future deployments.')


def ServiceAccountFlag():
  return StringFlag(
      '--service-account',
      help='Service account associated with the revision of the service. '
      'The service account represents the identity of the running revision, '
      'and determines what permissions the revision has. This is the name of '
      'a Kubernetes service account in the same namespace as the service. '
      'If not provided, the revision will use the default Kubernetes '
      'namespace service account. To reset this field to its default, pass an '
      'empty string.')


def TimeoutFlag():
  return StringFlag(
      '--timeout',
      help='Maximum request execution time (timeout). It is specified '
      'as a duration; for example, "10m5s" is ten minutes and five seconds. '
      'If you don\'t specify a unit, seconds is assumed. For example, "10" is '
      '10 seconds. Specify "0" to set the timeout to the default value.',
      type=arg_parsers.Duration(lower_bound='0s', parsed_unit='s'))


def RevisionSuffixFlag():
  return StringFlag(
      '--revision-suffix',
      help='Suffix of the revision name. Revision names always start with the '
      'service name automatically. For example, specifying '
      "`--revision-suffix=v1` for a service named 'helloworld', would lead "
      "to a revision named 'helloworld-v1'.")


def AsyncFlag():
  return BasicFlag(
      '--async',
      help='Return immediately, without waiting for the operation in progress '
      'to complete.')


def RegisterFlags(parser, flags):
  """Helper to register a set of flags with a given parser."""
  for f in flags:
    f.AddToParser(parser)


def _GetDestNameForFlag(flag):
  return flag.replace('--', '').replace('-', '_')


def _GetEnvironment(args):
  """Prompt for environment if not provided.

  Environment is decided in the following order:
  - environment argument;
  - kuberun/environment gcloud config;
  - prompt user.

  Args:
    args: Environment, The args environment.

  Returns:
    A str representing the environment name.

  Raises:
    A ValueError if no environment is specified.
  """
  env = None
  if getattr(args, 'environment', None):
    env = args.environment
  elif properties.VALUES.kuberun.environment.IsExplicitlySet():
    env = properties.VALUES.kuberun.environment.Get()
  elif console_io.CanPrompt():
    env = console_io.PromptWithDefault('Environment name', default=None)
    log.status.Print(
        'To make this the default environment, run '
        '`gcloud config set kuberun/environment {}`.\n'.format(env))
  if env:
    return env
  raise ValueError('Please specify an ENVIRONMENT to use this command.')


class EnvironmentFlag(StringFlag):
  """Formats an environment flag by pulling it from the appropriate source."""

  def __init__(self):
    super(EnvironmentFlag, self).__init__(
        '--environment',
        help='Name of the target KubeRun environment. '
             'Alternatively, set the property [kuberun/environment].')

  def FormatFlags(self, args):
    env = _GetEnvironment(args)
    return ['--environment', env]


def MaxRateFlag(required=False):
  return StringFlag(
      '--max-rate',
      help='Maximum number of requests per second that the backend can handle.',
      required=required)



"""Generated message classes for secretmanager version v1.

Stores sensitive data such as API keys, passwords, and certificates. Provides
convenience while improving security.
"""
# NOTE: This file is autogenerated and should not be edited by hand.

from __future__ import absolute_import

from apitools.base.protorpclite import messages as _messages
from apitools.base.py import encoding
from apitools.base.py import extra_types


package = 'secretmanager'


class AccessSecretVersionResponse(_messages.Message):
  r"""Response message for SecretManagerService.AccessSecretVersion.

  Fields:
    name: The resource name of the SecretVersion in the format
      `projects/*/secrets/*/versions/*`.
    payload: Secret payload
  """

  name = _messages.StringField(1)
  payload = _messages.MessageField('SecretPayload', 2)


class AddSecretVersionRequest(_messages.Message):
  r"""Request message for SecretManagerService.AddSecretVersion.

  Fields:
    payload: Required. The secret payload of the SecretVersion.
  """

  payload = _messages.MessageField('SecretPayload', 1)


class AuditConfig(_messages.Message):
  r"""Specifies the audit configuration for a service. The configuration
  determines which permission types are logged, and what identities, if any,
  are exempted from logging. An AuditConfig must have one or more
  AuditLogConfigs. If there are AuditConfigs for both `allServices` and a
  specific service, the union of the two AuditConfigs is used for that
  service: the log_types specified in each AuditConfig are enabled, and the
  exempted_members in each AuditLogConfig are exempted. Example Policy with
  multiple AuditConfigs: { "audit_configs": [ { "service": "allServices",
  "audit_log_configs": [ { "log_type": "DATA_READ", "exempted_members": [
  "user:jose@example.com" ] }, { "log_type": "DATA_WRITE" }, { "log_type":
  "ADMIN_READ" } ] }, { "service": "sampleservice.googleapis.com",
  "audit_log_configs": [ { "log_type": "DATA_READ" }, { "log_type":
  "DATA_WRITE", "exempted_members": [ "user:aliya@example.com" ] } ] } ] } For
  sampleservice, this policy enables DATA_READ, DATA_WRITE and ADMIN_READ
  logging. It also exempts `jose@example.com` from DATA_READ logging, and
  `aliya@example.com` from DATA_WRITE logging.

  Fields:
    auditLogConfigs: The configuration for logging of each type of permission.
    service: Specifies a service that will be enabled for audit logging. For
      example, `storage.googleapis.com`, `cloudsql.googleapis.com`.
      `allServices` is a special value that covers all services.
  """

  auditLogConfigs = _messages.MessageField('AuditLogConfig', 1, repeated=True)
  service = _messages.StringField(2)


class AuditLogConfig(_messages.Message):
  r"""Provides the configuration for logging a type of permissions. Example: {
  "audit_log_configs": [ { "log_type": "DATA_READ", "exempted_members": [
  "user:jose@example.com" ] }, { "log_type": "DATA_WRITE" } ] } This enables
  'DATA_READ' and 'DATA_WRITE' logging, while exempting jose@example.com from
  DATA_READ logging.

  Enums:
    LogTypeValueValuesEnum: The log type that this config enables.

  Fields:
    exemptedMembers: Specifies the identities that do not cause logging for
      this type of permission. Follows the same format of Binding.members.
    logType: The log type that this config enables.
  """

  class LogTypeValueValuesEnum(_messages.Enum):
    r"""The log type that this config enables.

    Values:
      LOG_TYPE_UNSPECIFIED: Default case. Should never be this.
      ADMIN_READ: Admin reads. Example: CloudIAM getIamPolicy
      DATA_WRITE: Data writes. Example: CloudSQL Users create
      DATA_READ: Data reads. Example: CloudSQL Users list
    """
    LOG_TYPE_UNSPECIFIED = 0
    ADMIN_READ = 1
    DATA_WRITE = 2
    DATA_READ = 3

  exemptedMembers = _messages.StringField(1, repeated=True)
  logType = _messages.EnumField('LogTypeValueValuesEnum', 2)


class Automatic(_messages.Message):
  r"""A replication policy that replicates the Secret payload without any
  restrictions.

  Fields:
    customerManagedEncryption: Optional. The customer-managed encryption
      configuration of the Secret. If no configuration is provided, Google-
      managed default encryption is used. Updates to the Secret encryption
      configuration only apply to SecretVersions added afterwards. They do not
      apply retroactively to existing SecretVersions.
  """

  customerManagedEncryption = _messages.MessageField('CustomerManagedEncryption', 1)


class AutomaticStatus(_messages.Message):
  r"""The replication status of a SecretVersion using automatic replication.
  Only populated if the parent Secret has an automatic replication policy.

  Fields:
    customerManagedEncryption: Output only. The customer-managed encryption
      status of the SecretVersion. Only populated if customer-managed
      encryption is used.
  """

  customerManagedEncryption = _messages.MessageField('CustomerManagedEncryptionStatus', 1)


class Binding(_messages.Message):
  r"""Associates `members`, or principals, with a `role`.

  Fields:
    condition: The condition that is associated with this binding. If the
      condition evaluates to `true`, then this binding applies to the current
      request. If the condition evaluates to `false`, then this binding does
      not apply to the current request. However, a different role binding
      might grant the same role to one or more of the principals in this
      binding. To learn which resources support conditions in their IAM
      policies, see the [IAM
      documentation](https://cloud.google.com/iam/help/conditions/resource-
      policies).
    members: Specifies the principals requesting access for a Google Cloud
      resource. `members` can have the following values: * `allUsers`: A
      special identifier that represents anyone who is on the internet; with
      or without a Google account. * `allAuthenticatedUsers`: A special
      identifier that represents anyone who is authenticated with a Google
      account or a service account. Does not include identities that come from
      external identity providers (IdPs) through identity federation. *
      `user:{emailid}`: An email address that represents a specific Google
      account. For example, `alice@example.com` . *
      `serviceAccount:{emailid}`: An email address that represents a Google
      service account. For example, `my-other-
      app@appspot.gserviceaccount.com`. *
      `serviceAccount:{projectid}.svc.id.goog[{namespace}/{kubernetes-sa}]`:
      An identifier for a [Kubernetes service
      account](https://cloud.google.com/kubernetes-engine/docs/how-
      to/kubernetes-service-accounts). For example, `my-
      project.svc.id.goog[my-namespace/my-kubernetes-sa]`. *
      `group:{emailid}`: An email address that represents a Google group. For
      example, `admins@example.com`. *
      `deleted:user:{emailid}?uid={uniqueid}`: An email address (plus unique
      identifier) representing a user that has been recently deleted. For
      example, `alice@example.com?uid=123456789012345678901`. If the user is
      recovered, this value reverts to `user:{emailid}` and the recovered user
      retains the role in the binding. *
      `deleted:serviceAccount:{emailid}?uid={uniqueid}`: An email address
      (plus unique identifier) representing a service account that has been
      recently deleted. For example, `my-other-
      app@appspot.gserviceaccount.com?uid=123456789012345678901`. If the
      service account is undeleted, this value reverts to
      `serviceAccount:{emailid}` and the undeleted service account retains the
      role in the binding. * `deleted:group:{emailid}?uid={uniqueid}`: An
      email address (plus unique identifier) representing a Google group that
      has been recently deleted. For example,
      `admins@example.com?uid=123456789012345678901`. If the group is
      recovered, this value reverts to `group:{emailid}` and the recovered
      group retains the role in the binding. * `domain:{domain}`: The G Suite
      domain (primary) that represents all the users of that domain. For
      example, `google.com` or `example.com`.
    role: Role that is assigned to the list of `members`, or principals. For
      example, `roles/viewer`, `roles/editor`, or `roles/owner`.
  """

  condition = _messages.MessageField('Expr', 1)
  members = _messages.StringField(2, repeated=True)
  role = _messages.StringField(3)


class CustomerManagedEncryption(_messages.Message):
  r"""Configuration for encrypting secret payloads using customer-managed
  encryption keys (CMEK).

  Fields:
    kmsKeyName: Required. The resource name of the Cloud KMS CryptoKey used to
      encrypt secret payloads. For secrets using the UserManaged replication
      policy type, Cloud KMS CryptoKeys must reside in the same location as
      the replica location. For secrets using the Automatic replication policy
      type, Cloud KMS CryptoKeys must reside in `global`. The expected format
      is `projects/*/locations/*/keyRings/*/cryptoKeys/*`.
  """

  kmsKeyName = _messages.StringField(1)


class CustomerManagedEncryptionStatus(_messages.Message):
  r"""Describes the status of customer-managed encryption.

  Fields:
    kmsKeyVersionName: Required. The resource name of the Cloud KMS
      CryptoKeyVersion used to encrypt the secret payload, in the following
      format: `projects/*/locations/*/keyRings/*/cryptoKeys/*/versions/*`.
  """

  kmsKeyVersionName = _messages.StringField(1)


class DestroySecretVersionRequest(_messages.Message):
  r"""Request message for SecretManagerService.DestroySecretVersion.

  Fields:
    etag: Optional. Etag of the SecretVersion. The request succeeds if it
      matches the etag of the currently stored secret version object. If the
      etag is omitted, the request succeeds.
  """

  etag = _messages.StringField(1)


class DisableSecretVersionRequest(_messages.Message):
  r"""Request message for SecretManagerService.DisableSecretVersion.

  Fields:
    etag: Optional. Etag of the SecretVersion. The request succeeds if it
      matches the etag of the currently stored secret version object. If the
      etag is omitted, the request succeeds.
  """

  etag = _messages.StringField(1)


class Empty(_messages.Message):
  r"""A generic empty message that you can re-use to avoid defining duplicated
  empty messages in your APIs. A typical example is to use it as the request
  or the response type of an API method. For instance: service Foo { rpc
  Bar(google.protobuf.Empty) returns (google.protobuf.Empty); }
  """



class EnableSecretVersionRequest(_messages.Message):
  r"""Request message for SecretManagerService.EnableSecretVersion.

  Fields:
    etag: Optional. Etag of the SecretVersion. The request succeeds if it
      matches the etag of the currently stored secret version object. If the
      etag is omitted, the request succeeds.
  """

  etag = _messages.StringField(1)


class Expr(_messages.Message):
  r"""Represents a textual expression in the Common Expression Language (CEL)
  syntax. CEL is a C-like expression language. The syntax and semantics of CEL
  are documented at https://github.com/google/cel-spec. Example (Comparison):
  title: "Summary size limit" description: "Determines if a summary is less
  than 100 chars" expression: "document.summary.size() < 100" Example
  (Equality): title: "Requestor is owner" description: "Determines if
  requestor is the document owner" expression: "document.owner ==
  request.auth.claims.email" Example (Logic): title: "Public documents"
  description: "Determine whether the document should be publicly visible"
  expression: "document.type != 'private' && document.type != 'internal'"
  Example (Data Manipulation): title: "Notification string" description:
  "Create a notification string with a timestamp." expression: "'New message
  received at ' + string(document.create_time)" The exact variables and
  functions that may be referenced within an expression are determined by the
  service that evaluates it. See the service documentation for additional
  information.

  Fields:
    description: Optional. Description of the expression. This is a longer
      text which describes the expression, e.g. when hovered over it in a UI.
    expression: Textual representation of an expression in Common Expression
      Language syntax.
    location: Optional. String indicating the location of the expression for
      error reporting, e.g. a file name and a position in the file.
    title: Optional. Title for the expression, i.e. a short string describing
      its purpose. This can be used e.g. in UIs which allow to enter the
      expression.
  """

  description = _messages.StringField(1)
  expression = _messages.StringField(2)
  location = _messages.StringField(3)
  title = _messages.StringField(4)


class ListLocationsResponse(_messages.Message):
  r"""The response message for Locations.ListLocations.

  Fields:
    locations: A list of locations that matches the specified filter in the
      request.
    nextPageToken: The standard List next-page token.
  """

  locations = _messages.MessageField('Location', 1, repeated=True)
  nextPageToken = _messages.StringField(2)


class ListSecretVersionsResponse(_messages.Message):
  r"""Response message for SecretManagerService.ListSecretVersions.

  Fields:
    nextPageToken: A token to retrieve the next page of results. Pass this
      value in ListSecretVersionsRequest.page_token to retrieve the next page.
    totalSize: The total number of SecretVersions but 0 when the
      ListSecretsRequest.filter field is set.
    versions: The list of SecretVersions sorted in reverse by create_time
      (newest first).
  """

  nextPageToken = _messages.StringField(1)
  totalSize = _messages.IntegerField(2, variant=_messages.Variant.INT32)
  versions = _messages.MessageField('SecretVersion', 3, repeated=True)


class ListSecretsResponse(_messages.Message):
  r"""Response message for SecretManagerService.ListSecrets.

  Fields:
    nextPageToken: A token to retrieve the next page of results. Pass this
      value in ListSecretsRequest.page_token to retrieve the next page.
    secrets: The list of Secrets sorted in reverse by create_time (newest
      first).
    totalSize: The total number of Secrets but 0 when the
      ListSecretsRequest.filter field is set.
  """

  nextPageToken = _messages.StringField(1)
  secrets = _messages.MessageField('Secret', 2, repeated=True)
  totalSize = _messages.IntegerField(3, variant=_messages.Variant.INT32)


class Location(_messages.Message):
  r"""A resource that represents Google Cloud Platform location.

  Messages:
    LabelsValue: Cross-service attributes for the location. For example
      {"cloud.googleapis.com/region": "us-east1"}
    MetadataValue: Service-specific metadata. For example the available
      capacity at the given location.

  Fields:
    displayName: The friendly name for this location, typically a nearby city
      name. For example, "Tokyo".
    labels: Cross-service attributes for the location. For example
      {"cloud.googleapis.com/region": "us-east1"}
    locationId: The canonical id for this location. For example: `"us-east1"`.
    metadata: Service-specific metadata. For example the available capacity at
      the given location.
    name: Resource name for the location, which may vary between
      implementations. For example: `"projects/example-project/locations/us-
      east1"`
  """

  @encoding.MapUnrecognizedFields('additionalProperties')
  class LabelsValue(_messages.Message):
    r"""Cross-service attributes for the location. For example
    {"cloud.googleapis.com/region": "us-east1"}

    Messages:
      AdditionalProperty: An additional property for a LabelsValue object.

    Fields:
      additionalProperties: Additional properties of type LabelsValue
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a LabelsValue object.

      Fields:
        key: Name of the additional property.
        value: A string attribute.
      """

      key = _messages.StringField(1)
      value = _messages.StringField(2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  @encoding.MapUnrecognizedFields('additionalProperties')
  class MetadataValue(_messages.Message):
    r"""Service-specific metadata. For example the available capacity at the
    given location.

    Messages:
      AdditionalProperty: An additional property for a MetadataValue object.

    Fields:
      additionalProperties: Properties of the object. Contains field @type
        with type URL.
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a MetadataValue object.

      Fields:
        key: Name of the additional property.
        value: A extra_types.JsonValue attribute.
      """

      key = _messages.StringField(1)
      value = _messages.MessageField('extra_types.JsonValue', 2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  displayName = _messages.StringField(1)
  labels = _messages.MessageField('LabelsValue', 2)
  locationId = _messages.StringField(3)
  metadata = _messages.MessageField('MetadataValue', 4)
  name = _messages.StringField(5)


class Policy(_messages.Message):
  r"""An Identity and Access Management (IAM) policy, which specifies access
  controls for Google Cloud resources. A `Policy` is a collection of
  `bindings`. A `binding` binds one or more `members`, or principals, to a
  single `role`. Principals can be user accounts, service accounts, Google
  groups, and domains (such as G Suite). A `role` is a named list of
  permissions; each `role` can be an IAM predefined role or a user-created
  custom role. For some types of Google Cloud resources, a `binding` can also
  specify a `condition`, which is a logical expression that allows access to a
  resource only if the expression evaluates to `true`. A condition can add
  constraints based on attributes of the request, the resource, or both. To
  learn which resources support conditions in their IAM policies, see the [IAM
  documentation](https://cloud.google.com/iam/help/conditions/resource-
  policies). **JSON example:** { "bindings": [ { "role":
  "roles/resourcemanager.organizationAdmin", "members": [
  "user:mike@example.com", "group:admins@example.com", "domain:google.com",
  "serviceAccount:my-project-id@appspot.gserviceaccount.com" ] }, { "role":
  "roles/resourcemanager.organizationViewer", "members": [
  "user:eve@example.com" ], "condition": { "title": "expirable access",
  "description": "Does not grant access after Sep 2020", "expression":
  "request.time < timestamp('2020-10-01T00:00:00.000Z')", } } ], "etag":
  "BwWWja0YfJA=", "version": 3 } **YAML example:** bindings: - members: -
  user:mike@example.com - group:admins@example.com - domain:google.com -
  serviceAccount:my-project-id@appspot.gserviceaccount.com role:
  roles/resourcemanager.organizationAdmin - members: - user:eve@example.com
  role: roles/resourcemanager.organizationViewer condition: title: expirable
  access description: Does not grant access after Sep 2020 expression:
  request.time < timestamp('2020-10-01T00:00:00.000Z') etag: BwWWja0YfJA=
  version: 3 For a description of IAM and its features, see the [IAM
  documentation](https://cloud.google.com/iam/docs/).

  Fields:
    auditConfigs: Specifies cloud audit logging configuration for this policy.
    bindings: Associates a list of `members`, or principals, with a `role`.
      Optionally, may specify a `condition` that determines how and when the
      `bindings` are applied. Each of the `bindings` must contain at least one
      principal. The `bindings` in a `Policy` can refer to up to 1,500
      principals; up to 250 of these principals can be Google groups. Each
      occurrence of a principal counts towards these limits. For example, if
      the `bindings` grant 50 different roles to `user:alice@example.com`, and
      not to any other principal, then you can add another 1,450 principals to
      the `bindings` in the `Policy`.
    etag: `etag` is used for optimistic concurrency control as a way to help
      prevent simultaneous updates of a policy from overwriting each other. It
      is strongly suggested that systems make use of the `etag` in the read-
      modify-write cycle to perform policy updates in order to avoid race
      conditions: An `etag` is returned in the response to `getIamPolicy`, and
      systems are expected to put that etag in the request to `setIamPolicy`
      to ensure that their change will be applied to the same version of the
      policy. **Important:** If you use IAM Conditions, you must include the
      `etag` field whenever you call `setIamPolicy`. If you omit this field,
      then IAM allows you to overwrite a version `3` policy with a version `1`
      policy, and all of the conditions in the version `3` policy are lost.
    version: Specifies the format of the policy. Valid values are `0`, `1`,
      and `3`. Requests that specify an invalid value are rejected. Any
      operation that affects conditional role bindings must specify version
      `3`. This requirement applies to the following operations: * Getting a
      policy that includes a conditional role binding * Adding a conditional
      role binding to a policy * Changing a conditional role binding in a
      policy * Removing any role binding, with or without a condition, from a
      policy that includes conditions **Important:** If you use IAM
      Conditions, you must include the `etag` field whenever you call
      `setIamPolicy`. If you omit this field, then IAM allows you to overwrite
      a version `3` policy with a version `1` policy, and all of the
      conditions in the version `3` policy are lost. If a policy does not
      include any conditions, operations on that policy may specify any valid
      version or leave the field unset. To learn which resources support
      conditions in their IAM policies, see the [IAM
      documentation](https://cloud.google.com/iam/help/conditions/resource-
      policies).
  """

  auditConfigs = _messages.MessageField('AuditConfig', 1, repeated=True)
  bindings = _messages.MessageField('Binding', 2, repeated=True)
  etag = _messages.BytesField(3)
  version = _messages.IntegerField(4, variant=_messages.Variant.INT32)


class Replica(_messages.Message):
  r"""Represents a Replica for this Secret.

  Fields:
    customerManagedEncryption: Optional. The customer-managed encryption
      configuration of the User-Managed Replica. If no configuration is
      provided, Google-managed default encryption is used. Updates to the
      Secret encryption configuration only apply to SecretVersions added
      afterwards. They do not apply retroactively to existing SecretVersions.
    location: The canonical IDs of the location to replicate data. For
      example: `"us-east1"`.
  """

  customerManagedEncryption = _messages.MessageField('CustomerManagedEncryption', 1)
  location = _messages.StringField(2)


class ReplicaStatus(_messages.Message):
  r"""Describes the status of a user-managed replica for the SecretVersion.

  Fields:
    customerManagedEncryption: Output only. The customer-managed encryption
      status of the SecretVersion. Only populated if customer-managed
      encryption is used.
    location: Output only. The canonical ID of the replica location. For
      example: `"us-east1"`.
  """

  customerManagedEncryption = _messages.MessageField('CustomerManagedEncryptionStatus', 1)
  location = _messages.StringField(2)


class Replication(_messages.Message):
  r"""A policy that defines the replication and encryption configuration of
  data.

  Fields:
    automatic: The Secret will automatically be replicated without any
      restrictions.
    userManaged: The Secret will only be replicated into the locations
      specified.
  """

  automatic = _messages.MessageField('Automatic', 1)
  userManaged = _messages.MessageField('UserManaged', 2)


class ReplicationStatus(_messages.Message):
  r"""The replication status of a SecretVersion.

  Fields:
    automatic: Describes the replication status of a SecretVersion with
      automatic replication. Only populated if the parent Secret has an
      automatic replication policy.
    userManaged: Describes the replication status of a SecretVersion with
      user-managed replication. Only populated if the parent Secret has a
      user-managed replication policy.
  """

  automatic = _messages.MessageField('AutomaticStatus', 1)
  userManaged = _messages.MessageField('UserManagedStatus', 2)


class Rotation(_messages.Message):
  r"""The rotation time and period for a Secret. At next_rotation_time, Secret
  Manager will send a Pub/Sub notification to the topics configured on the
  Secret. Secret.topics must be set to configure rotation.

  Fields:
    nextRotationTime: Optional. Timestamp in UTC at which the Secret is
      scheduled to rotate. Cannot be set to less than 300s (5 min) in the
      future and at most 3153600000s (100 years). next_rotation_time MUST be
      set if rotation_period is set.
    rotationPeriod: Input only. The Duration between rotation notifications.
      Must be in seconds and at least 3600s (1h) and at most 3153600000s (100
      years). If rotation_period is set, next_rotation_time must be set.
      next_rotation_time will be advanced by this period when the service
      automatically sends rotation notifications.
  """

  nextRotationTime = _messages.StringField(1)
  rotationPeriod = _messages.StringField(2)


class Secret(_messages.Message):
  r"""A Secret is a logical secret whose value and versions can be accessed. A
  Secret is made up of zero or more SecretVersions that represent the secret
  data.

  Messages:
    AnnotationsValue: Optional. Custom metadata about the secret. Annotations
      are distinct from various forms of labels. Annotations exist to allow
      client tools to store their own state information without requiring a
      database. Annotation keys must be between 1 and 63 characters long, have
      a UTF-8 encoding of maximum 128 bytes, begin and end with an
      alphanumeric character ([a-z0-9A-Z]), and may have dashes (-),
      underscores (_), dots (.), and alphanumerics in between these symbols.
      The total size of annotation keys and values must be less than 16KiB.
    LabelsValue: The labels assigned to this Secret. Label keys must be
      between 1 and 63 characters long, have a UTF-8 encoding of maximum 128
      bytes, and must conform to the following PCRE regular expression:
      `\p{Ll}\p{Lo}{0,62}` Label values must be between 0 and 63 characters
      long, have a UTF-8 encoding of maximum 128 bytes, and must conform to
      the following PCRE regular expression: `[\p{Ll}\p{Lo}\p{N}_-]{0,63}` No
      more than 64 labels can be assigned to a given resource.
    VersionAliasesValue: Optional. Mapping from version alias to version name.
      A version alias is a string with a maximum length of 63 characters and
      can contain uppercase and lowercase letters, numerals, and the hyphen
      (`-`) and underscore ('_') characters. An alias string must start with a
      letter and cannot be the string 'latest' or 'NEW'. No more than 50
      aliases can be assigned to a given secret. Version-Alias pairs will be
      viewable via GetSecret and modifiable via UpdateSecret. At launch Access
      by Allias will only be supported on GetSecretVersion and
      AccessSecretVersion.

  Fields:
    annotations: Optional. Custom metadata about the secret. Annotations are
      distinct from various forms of labels. Annotations exist to allow client
      tools to store their own state information without requiring a database.
      Annotation keys must be between 1 and 63 characters long, have a UTF-8
      encoding of maximum 128 bytes, begin and end with an alphanumeric
      character ([a-z0-9A-Z]), and may have dashes (-), underscores (_), dots
      (.), and alphanumerics in between these symbols. The total size of
      annotation keys and values must be less than 16KiB.
    createTime: Output only. The time at which the Secret was created.
    etag: Optional. Etag of the currently stored Secret.
    expireTime: Optional. Timestamp in UTC when the Secret is scheduled to
      expire. This is always provided on output, regardless of what was sent
      on input.
    labels: The labels assigned to this Secret. Label keys must be between 1
      and 63 characters long, have a UTF-8 encoding of maximum 128 bytes, and
      must conform to the following PCRE regular expression:
      `\p{Ll}\p{Lo}{0,62}` Label values must be between 0 and 63 characters
      long, have a UTF-8 encoding of maximum 128 bytes, and must conform to
      the following PCRE regular expression: `[\p{Ll}\p{Lo}\p{N}_-]{0,63}` No
      more than 64 labels can be assigned to a given resource.
    name: Output only. The resource name of the Secret in the format
      `projects/*/secrets/*`.
    replication: Required. Immutable. The replication policy of the secret
      data attached to the Secret. The replication policy cannot be changed
      after the Secret has been created.
    rotation: Optional. Rotation policy attached to the Secret. May be
      excluded if there is no rotation policy.
    topics: Optional. A list of up to 10 Pub/Sub topics to which messages are
      published when control plane operations are called on the secret or its
      versions.
    ttl: Input only. The TTL for the Secret.
    versionAliases: Optional. Mapping from version alias to version name. A
      version alias is a string with a maximum length of 63 characters and can
      contain uppercase and lowercase letters, numerals, and the hyphen (`-`)
      and underscore ('_') characters. An alias string must start with a
      letter and cannot be the string 'latest' or 'NEW'. No more than 50
      aliases can be assigned to a given secret. Version-Alias pairs will be
      viewable via GetSecret and modifiable via UpdateSecret. At launch Access
      by Allias will only be supported on GetSecretVersion and
      AccessSecretVersion.
  """

  @encoding.MapUnrecognizedFields('additionalProperties')
  class AnnotationsValue(_messages.Message):
    r"""Optional. Custom metadata about the secret. Annotations are distinct
    from various forms of labels. Annotations exist to allow client tools to
    store their own state information without requiring a database. Annotation
    keys must be between 1 and 63 characters long, have a UTF-8 encoding of
    maximum 128 bytes, begin and end with an alphanumeric character
    ([a-z0-9A-Z]), and may have dashes (-), underscores (_), dots (.), and
    alphanumerics in between these symbols. The total size of annotation keys
    and values must be less than 16KiB.

    Messages:
      AdditionalProperty: An additional property for a AnnotationsValue
        object.

    Fields:
      additionalProperties: Additional properties of type AnnotationsValue
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a AnnotationsValue object.

      Fields:
        key: Name of the additional property.
        value: A string attribute.
      """

      key = _messages.StringField(1)
      value = _messages.StringField(2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  @encoding.MapUnrecognizedFields('additionalProperties')
  class LabelsValue(_messages.Message):
    r"""The labels assigned to this Secret. Label keys must be between 1 and
    63 characters long, have a UTF-8 encoding of maximum 128 bytes, and must
    conform to the following PCRE regular expression: `\p{Ll}\p{Lo}{0,62}`
    Label values must be between 0 and 63 characters long, have a UTF-8
    encoding of maximum 128 bytes, and must conform to the following PCRE
    regular expression: `[\p{Ll}\p{Lo}\p{N}_-]{0,63}` No more than 64 labels
    can be assigned to a given resource.

    Messages:
      AdditionalProperty: An additional property for a LabelsValue object.

    Fields:
      additionalProperties: Additional properties of type LabelsValue
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a LabelsValue object.

      Fields:
        key: Name of the additional property.
        value: A string attribute.
      """

      key = _messages.StringField(1)
      value = _messages.StringField(2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  @encoding.MapUnrecognizedFields('additionalProperties')
  class VersionAliasesValue(_messages.Message):
    r"""Optional. Mapping from version alias to version name. A version alias
    is a string with a maximum length of 63 characters and can contain
    uppercase and lowercase letters, numerals, and the hyphen (`-`) and
    underscore ('_') characters. An alias string must start with a letter and
    cannot be the string 'latest' or 'NEW'. No more than 50 aliases can be
    assigned to a given secret. Version-Alias pairs will be viewable via
    GetSecret and modifiable via UpdateSecret. At launch Access by Allias will
    only be supported on GetSecretVersion and AccessSecretVersion.

    Messages:
      AdditionalProperty: An additional property for a VersionAliasesValue
        object.

    Fields:
      additionalProperties: Additional properties of type VersionAliasesValue
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a VersionAliasesValue object.

      Fields:
        key: Name of the additional property.
        value: A string attribute.
      """

      key = _messages.StringField(1)
      value = _messages.IntegerField(2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  annotations = _messages.MessageField('AnnotationsValue', 1)
  createTime = _messages.StringField(2)
  etag = _messages.StringField(3)
  expireTime = _messages.StringField(4)
  labels = _messages.MessageField('LabelsValue', 5)
  name = _messages.StringField(6)
  replication = _messages.MessageField('Replication', 7)
  rotation = _messages.MessageField('Rotation', 8)
  topics = _messages.MessageField('Topic', 9, repeated=True)
  ttl = _messages.StringField(10)
  versionAliases = _messages.MessageField('VersionAliasesValue', 11)


class SecretPayload(_messages.Message):
  r"""A secret payload resource in the Secret Manager API. This contains the
  sensitive secret payload that is associated with a SecretVersion.

  Fields:
    data: The secret data. Must be no larger than 64KiB.
    dataCrc32c: Optional. If specified, SecretManagerService will verify the
      integrity of the received data on SecretManagerService.AddSecretVersion
      calls using the crc32c checksum and store it to include in future
      SecretManagerService.AccessSecretVersion responses. If a checksum is not
      provided in the SecretManagerService.AddSecretVersion request, the
      SecretManagerService will generate and store one for you. The CRC32C
      value is encoded as a Int64 for compatibility, and can be safely
      downconverted to uint32 in languages that support this type.
      https://cloud.google.com/apis/design/design_patterns#integer_types
  """

  data = _messages.BytesField(1)
  dataCrc32c = _messages.IntegerField(2)


class SecretVersion(_messages.Message):
  r"""A secret version resource in the Secret Manager API.

  Enums:
    StateValueValuesEnum: Output only. The current state of the SecretVersion.

  Fields:
    clientSpecifiedPayloadChecksum: Output only. True if payload checksum
      specified in SecretPayload object has been received by
      SecretManagerService on SecretManagerService.AddSecretVersion.
    createTime: Output only. The time at which the SecretVersion was created.
    destroyTime: Output only. The time this SecretVersion was destroyed. Only
      present if state is DESTROYED.
    etag: Output only. Etag of the currently stored SecretVersion.
    name: Output only. The resource name of the SecretVersion in the format
      `projects/*/secrets/*/versions/*`. SecretVersion IDs in a Secret start
      at 1 and are incremented for each subsequent version of the secret.
    replicationStatus: The replication status of the SecretVersion.
    state: Output only. The current state of the SecretVersion.
  """

  class StateValueValuesEnum(_messages.Enum):
    r"""Output only. The current state of the SecretVersion.

    Values:
      STATE_UNSPECIFIED: Not specified. This value is unused and invalid.
      ENABLED: The SecretVersion may be accessed.
      DISABLED: The SecretVersion may not be accessed, but the secret data is
        still available and can be placed back into the ENABLED state.
      DESTROYED: The SecretVersion is destroyed and the secret data is no
        longer stored. A version may not leave this state once entered.
    """
    STATE_UNSPECIFIED = 0
    ENABLED = 1
    DISABLED = 2
    DESTROYED = 3

  clientSpecifiedPayloadChecksum = _messages.BooleanField(1)
  createTime = _messages.StringField(2)
  destroyTime = _messages.StringField(3)
  etag = _messages.StringField(4)
  name = _messages.StringField(5)
  replicationStatus = _messages.MessageField('ReplicationStatus', 6)
  state = _messages.EnumField('StateValueValuesEnum', 7)


class SecretmanagerProjectsLocationsGetRequest(_messages.Message):
  r"""A SecretmanagerProjectsLocationsGetRequest object.

  Fields:
    name: Resource name for the location.
  """

  name = _messages.StringField(1, required=True)


class SecretmanagerProjectsLocationsListRequest(_messages.Message):
  r"""A SecretmanagerProjectsLocationsListRequest object.

  Fields:
    filter: A filter to narrow down results to a preferred subset. The
      filtering language accepts strings like `"displayName=tokyo"`, and is
      documented in more detail in [AIP-160](https://google.aip.dev/160).
    name: The resource that owns the locations collection, if applicable.
    pageSize: The maximum number of results to return. If not set, the service
      selects a default.
    pageToken: A page token received from the `next_page_token` field in the
      response. Send that page token to receive the subsequent page.
  """

  filter = _messages.StringField(1)
  name = _messages.StringField(2, required=True)
  pageSize = _messages.IntegerField(3, variant=_messages.Variant.INT32)
  pageToken = _messages.StringField(4)


class SecretmanagerProjectsSecretsAddVersionRequest(_messages.Message):
  r"""A SecretmanagerProjectsSecretsAddVersionRequest object.

  Fields:
    addSecretVersionRequest: A AddSecretVersionRequest resource to be passed
      as the request body.
    parent: Required. The resource name of the Secret to associate with the
      SecretVersion in the format `projects/*/secrets/*`.
  """

  addSecretVersionRequest = _messages.MessageField('AddSecretVersionRequest', 1)
  parent = _messages.StringField(2, required=True)


class SecretmanagerProjectsSecretsCreateRequest(_messages.Message):
  r"""A SecretmanagerProjectsSecretsCreateRequest object.

  Fields:
    parent: Required. The resource name of the project to associate with the
      Secret, in the format `projects/*`.
    secret: A Secret resource to be passed as the request body.
    secretId: Required. This must be unique within the project. A secret ID is
      a string with a maximum length of 255 characters and can contain
      uppercase and lowercase letters, numerals, and the hyphen (`-`) and
      underscore (`_`) characters.
  """

  parent = _messages.StringField(1, required=True)
  secret = _messages.MessageField('Secret', 2)
  secretId = _messages.StringField(3)


class SecretmanagerProjectsSecretsDeleteRequest(_messages.Message):
  r"""A SecretmanagerProjectsSecretsDeleteRequest object.

  Fields:
    etag: Optional. Etag of the Secret. The request succeeds if it matches the
      etag of the currently stored secret object. If the etag is omitted, the
      request succeeds.
    name: Required. The resource name of the Secret to delete in the format
      `projects/*/secrets/*`.
  """

  etag = _messages.StringField(1)
  name = _messages.StringField(2, required=True)


class SecretmanagerProjectsSecretsGetIamPolicyRequest(_messages.Message):
  r"""A SecretmanagerProjectsSecretsGetIamPolicyRequest object.

  Fields:
    options_requestedPolicyVersion: Optional. The maximum policy version that
      will be used to format the policy. Valid values are 0, 1, and 3.
      Requests specifying an invalid value will be rejected. Requests for
      policies with any conditional role bindings must specify version 3.
      Policies with no conditional role bindings may specify any valid value
      or leave the field unset. The policy in the response might use the
      policy version that you specified, or it might use a lower policy
      version. For example, if you specify version 3, but the policy has no
      conditional role bindings, the response uses version 1. To learn which
      resources support conditions in their IAM policies, see the [IAM
      documentation](https://cloud.google.com/iam/help/conditions/resource-
      policies).
    resource: REQUIRED: The resource for which the policy is being requested.
      See [Resource
      names](https://cloud.google.com/apis/design/resource_names) for the
      appropriate value for this field.
  """

  options_requestedPolicyVersion = _messages.IntegerField(1, variant=_messages.Variant.INT32)
  resource = _messages.StringField(2, required=True)


class SecretmanagerProjectsSecretsGetRequest(_messages.Message):
  r"""A SecretmanagerProjectsSecretsGetRequest object.

  Fields:
    name: Required. The resource name of the Secret, in the format
      `projects/*/secrets/*`.
  """

  name = _messages.StringField(1, required=True)


class SecretmanagerProjectsSecretsListRequest(_messages.Message):
  r"""A SecretmanagerProjectsSecretsListRequest object.

  Fields:
    filter: Optional. Filter string, adhering to the rules in [List-operation
      filtering](https://cloud.google.com/secret-manager/docs/filtering). List
      only secrets matching the filter. If filter is empty, all secrets are
      listed.
    pageSize: Optional. The maximum number of results to be returned in a
      single page. If set to 0, the server decides the number of results to
      return. If the number is greater than 25000, it is capped at 25000.
    pageToken: Optional. Pagination token, returned earlier via
      ListSecretsResponse.next_page_token.
    parent: Required. The resource name of the project associated with the
      Secrets, in the format `projects/*`.
  """

  filter = _messages.StringField(1)
  pageSize = _messages.IntegerField(2, variant=_messages.Variant.INT32)
  pageToken = _messages.StringField(3)
  parent = _messages.StringField(4, required=True)


class SecretmanagerProjectsSecretsPatchRequest(_messages.Message):
  r"""A SecretmanagerProjectsSecretsPatchRequest object.

  Fields:
    name: Output only. The resource name of the Secret in the format
      `projects/*/secrets/*`.
    secret: A Secret resource to be passed as the request body.
    updateMask: Required. Specifies the fields to be updated.
  """

  name = _messages.StringField(1, required=True)
  secret = _messages.MessageField('Secret', 2)
  updateMask = _messages.StringField(3)


class SecretmanagerProjectsSecretsSetIamPolicyRequest(_messages.Message):
  r"""A SecretmanagerProjectsSecretsSetIamPolicyRequest object.

  Fields:
    resource: REQUIRED: The resource for which the policy is being specified.
      See [Resource
      names](https://cloud.google.com/apis/design/resource_names) for the
      appropriate value for this field.
    setIamPolicyRequest: A SetIamPolicyRequest resource to be passed as the
      request body.
  """

  resource = _messages.StringField(1, required=True)
  setIamPolicyRequest = _messages.MessageField('SetIamPolicyRequest', 2)


class SecretmanagerProjectsSecretsTestIamPermissionsRequest(_messages.Message):
  r"""A SecretmanagerProjectsSecretsTestIamPermissionsRequest object.

  Fields:
    resource: REQUIRED: The resource for which the policy detail is being
      requested. See [Resource
      names](https://cloud.google.com/apis/design/resource_names) for the
      appropriate value for this field.
    testIamPermissionsRequest: A TestIamPermissionsRequest resource to be
      passed as the request body.
  """

  resource = _messages.StringField(1, required=True)
  testIamPermissionsRequest = _messages.MessageField('TestIamPermissionsRequest', 2)


class SecretmanagerProjectsSecretsVersionsAccessRequest(_messages.Message):
  r"""A SecretmanagerProjectsSecretsVersionsAccessRequest object.

  Fields:
    name: Required. The resource name of the SecretVersion in the format
      `projects/*/secrets/*/versions/*`.
      `projects/*/secrets/*/versions/latest` is an alias to the most recently
      created SecretVersion.
  """

  name = _messages.StringField(1, required=True)


class SecretmanagerProjectsSecretsVersionsDestroyRequest(_messages.Message):
  r"""A SecretmanagerProjectsSecretsVersionsDestroyRequest object.

  Fields:
    destroySecretVersionRequest: A DestroySecretVersionRequest resource to be
      passed as the request body.
    name: Required. The resource name of the SecretVersion to destroy in the
      format `projects/*/secrets/*/versions/*`.
  """

  destroySecretVersionRequest = _messages.MessageField('DestroySecretVersionRequest', 1)
  name = _messages.StringField(2, required=True)


class SecretmanagerProjectsSecretsVersionsDisableRequest(_messages.Message):
  r"""A SecretmanagerProjectsSecretsVersionsDisableRequest object.

  Fields:
    disableSecretVersionRequest: A DisableSecretVersionRequest resource to be
      passed as the request body.
    name: Required. The resource name of the SecretVersion to disable in the
      format `projects/*/secrets/*/versions/*`.
  """

  disableSecretVersionRequest = _messages.MessageField('DisableSecretVersionRequest', 1)
  name = _messages.StringField(2, required=True)


class SecretmanagerProjectsSecretsVersionsEnableRequest(_messages.Message):
  r"""A SecretmanagerProjectsSecretsVersionsEnableRequest object.

  Fields:
    enableSecretVersionRequest: A EnableSecretVersionRequest resource to be
      passed as the request body.
    name: Required. The resource name of the SecretVersion to enable in the
      format `projects/*/secrets/*/versions/*`.
  """

  enableSecretVersionRequest = _messages.MessageField('EnableSecretVersionRequest', 1)
  name = _messages.StringField(2, required=True)


class SecretmanagerProjectsSecretsVersionsGetRequest(_messages.Message):
  r"""A SecretmanagerProjectsSecretsVersionsGetRequest object.

  Fields:
    name: Required. The resource name of the SecretVersion in the format
      `projects/*/secrets/*/versions/*`.
      `projects/*/secrets/*/versions/latest` is an alias to the most recently
      created SecretVersion.
  """

  name = _messages.StringField(1, required=True)


class SecretmanagerProjectsSecretsVersionsListRequest(_messages.Message):
  r"""A SecretmanagerProjectsSecretsVersionsListRequest object.

  Fields:
    filter: Optional. Filter string, adhering to the rules in [List-operation
      filtering](https://cloud.google.com/secret-manager/docs/filtering). List
      only secret versions matching the filter. If filter is empty, all secret
      versions are listed.
    pageSize: Optional. The maximum number of results to be returned in a
      single page. If set to 0, the server decides the number of results to
      return. If the number is greater than 25000, it is capped at 25000.
    pageToken: Optional. Pagination token, returned earlier via
      ListSecretVersionsResponse.next_page_token][].
    parent: Required. The resource name of the Secret associated with the
      SecretVersions to list, in the format `projects/*/secrets/*`.
  """

  filter = _messages.StringField(1)
  pageSize = _messages.IntegerField(2, variant=_messages.Variant.INT32)
  pageToken = _messages.StringField(3)
  parent = _messages.StringField(4, required=True)


class SetIamPolicyRequest(_messages.Message):
  r"""Request message for `SetIamPolicy` method.

  Fields:
    policy: REQUIRED: The complete policy to be applied to the `resource`. The
      size of the policy is limited to a few 10s of KB. An empty policy is a
      valid policy but certain Google Cloud services (such as Projects) might
      reject them.
    updateMask: OPTIONAL: A FieldMask specifying which fields of the policy to
      modify. Only the fields in the mask will be modified. If no mask is
      provided, the following default mask is used: `paths: "bindings, etag"`
  """

  policy = _messages.MessageField('Policy', 1)
  updateMask = _messages.StringField(2)


class StandardQueryParameters(_messages.Message):
  r"""Query parameters accepted by all methods.

  Enums:
    FXgafvValueValuesEnum: V1 error format.
    AltValueValuesEnum: Data format for response.

  Fields:
    f__xgafv: V1 error format.
    access_token: OAuth access token.
    alt: Data format for response.
    callback: JSONP
    fields: Selector specifying which fields to include in a partial response.
    key: API key. Your API key identifies your project and provides you with
      API access, quota, and reports. Required unless you provide an OAuth 2.0
      token.
    oauth_token: OAuth 2.0 token for the current user.
    prettyPrint: Returns response with indentations and line breaks.
    quotaUser: Available to use for quota purposes for server-side
      applications. Can be any arbitrary string assigned to a user, but should
      not exceed 40 characters.
    trace: A tracing token of the form "token:<tokenid>" to include in api
      requests.
    uploadType: Legacy upload protocol for media (e.g. "media", "multipart").
    upload_protocol: Upload protocol for media (e.g. "raw", "multipart").
  """

  class AltValueValuesEnum(_messages.Enum):
    r"""Data format for response.

    Values:
      json: Responses with Content-Type of application/json
      media: Media download with context-dependent Content-Type
      proto: Responses with Content-Type of application/x-protobuf
    """
    json = 0
    media = 1
    proto = 2

  class FXgafvValueValuesEnum(_messages.Enum):
    r"""V1 error format.

    Values:
      _1: v1 error format
      _2: v2 error format
    """
    _1 = 0
    _2 = 1

  f__xgafv = _messages.EnumField('FXgafvValueValuesEnum', 1)
  access_token = _messages.StringField(2)
  alt = _messages.EnumField('AltValueValuesEnum', 3, default='json')
  callback = _messages.StringField(4)
  fields = _messages.StringField(5)
  key = _messages.StringField(6)
  oauth_token = _messages.StringField(7)
  prettyPrint = _messages.BooleanField(8, default=True)
  quotaUser = _messages.StringField(9)
  trace = _messages.StringField(10)
  uploadType = _messages.StringField(11)
  upload_protocol = _messages.StringField(12)


class TestIamPermissionsRequest(_messages.Message):
  r"""Request message for `TestIamPermissions` method.

  Fields:
    permissions: The set of permissions to check for the `resource`.
      Permissions with wildcards (such as `*` or `storage.*`) are not allowed.
      For more information see [IAM
      Overview](https://cloud.google.com/iam/docs/overview#permissions).
  """

  permissions = _messages.StringField(1, repeated=True)


class TestIamPermissionsResponse(_messages.Message):
  r"""Response message for `TestIamPermissions` method.

  Fields:
    permissions: A subset of `TestPermissionsRequest.permissions` that the
      caller is allowed.
  """

  permissions = _messages.StringField(1, repeated=True)


class Topic(_messages.Message):
  r"""A Pub/Sub topic which Secret Manager will publish to when control plane
  events occur on this secret.

  Fields:
    name: Required. The resource name of the Pub/Sub topic that will be
      published to, in the following format: `projects/*/topics/*`. For
      publication to succeed, the Secret Manager P4SA must have
      `pubsub.publisher` permissions on the topic.
  """

  name = _messages.StringField(1)


class UserManaged(_messages.Message):
  r"""A replication policy that replicates the Secret payload into the
  locations specified in Secret.replication.user_managed.replicas

  Fields:
    replicas: Required. The list of Replicas for this Secret. Cannot be empty.
  """

  replicas = _messages.MessageField('Replica', 1, repeated=True)


class UserManagedStatus(_messages.Message):
  r"""The replication status of a SecretVersion using user-managed
  replication. Only populated if the parent Secret has a user-managed
  replication policy.

  Fields:
    replicas: Output only. The list of replica statuses for the SecretVersion.
  """

  replicas = _messages.MessageField('ReplicaStatus', 1, repeated=True)


encoding.AddCustomJsonFieldMapping(
    StandardQueryParameters, 'f__xgafv', '$.xgafv')
encoding.AddCustomJsonEnumMapping(
    StandardQueryParameters.FXgafvValueValuesEnum, '_1', '1')
encoding.AddCustomJsonEnumMapping(
    StandardQueryParameters.FXgafvValueValuesEnum, '_2', '2')
